package de.draegerit.jsfjaasauth;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;

import javax.security.auth.Subject;
import javax.security.auth.callback.Callback;
import javax.security.auth.callback.CallbackHandler;
import javax.security.auth.callback.NameCallback;
import javax.security.auth.callback.PasswordCallback;
import javax.security.auth.callback.UnsupportedCallbackException;
import javax.security.auth.login.FailedLoginException;
import javax.security.auth.login.LoginException;
import javax.security.auth.spi.LoginModule;
import javax.xml.bind.DatatypeConverter;

public class LoginModuleImpl implements LoginModule {

	private static final String PASSWORD = "password";
	private static final String LOGIN = "login";

	private static final String USER_GROUP_ADMIN = "admin";

	private Subject subject;
	private UserPrincipal userPrincipal;
	private RolePrincipal rolePrincipal;
	private String login;
	private List<String> userGroups = Collections.emptyList();
	private CallbackHandler callbackHandler;

	private Hashtable<String, String> userTable;
	private String userFileName;

	@Override
	public void initialize(Subject subject, CallbackHandler callbackHandler, Map<String, ?> sharedState,
			Map<String, ?> options) {
		this.callbackHandler = callbackHandler;
		this.subject = subject;

		userFileName = (String) options.get("userfile");
		assert (userFileName == null);
		try {
			userTable = readUserFile();
		} catch (LoginException e) {
			e.printStackTrace();
		}
	}

	@Override
	public boolean login() throws LoginException {
		Callback[] callbacks = new Callback[] { new NameCallback(LOGIN), new PasswordCallback(PASSWORD, true) };

		try {
			callbackHandler.handle(callbacks);
			String name = ((NameCallback) callbacks[0]).getName();
			String password = String.valueOf(((PasswordCallback) callbacks[1]).getPassword());

			if (!userTable.isEmpty()) {
				String regUserPw = userTable.get(name);
				if (regUserPw == null) {
					throw new LoginException("Benutzer [" + name + "] wurde nicht gefunden!");
				} else if (regUserPw != null && regUserPw.equals(generateHashValue(password))) {
					login = name;
					userGroups = new ArrayList<String>();
					userGroups.add(USER_GROUP_ADMIN);
					return true;
				} else {
					throw new FailedLoginException("Authentifizierung fehlgeschlagen!");
				}

			} else {
				throw new LoginException("Es wurden keine Benutzer eingerichtet!");
			}
		} catch (IOException | UnsupportedCallbackException | NoSuchAlgorithmException e) {
			throw new LoginException(e.getMessage());
		}
	}

	private String generateHashValue(String pw) throws NoSuchAlgorithmException, UnsupportedEncodingException {
		MessageDigest md = MessageDigest.getInstance("MD5");
		md.update(pw.getBytes());
		byte[] digest = md.digest();
		return DatatypeConverter.printHexBinary(digest);
	}

	@Override
	public boolean commit() throws LoginException {
		userPrincipal = new UserPrincipal(login);
		subject.getPrincipals().add(userPrincipal);

		for (String groupName : userGroups) {
			rolePrincipal = new RolePrincipal(groupName);
			subject.getPrincipals().add(rolePrincipal);
		}

		return true;
	}

	@Override
	public boolean abort() throws LoginException {
		return false;
	}

	@Override
	public boolean logout() throws LoginException {
		subject.getPrincipals().remove(userPrincipal);
		subject.getPrincipals().remove(rolePrincipal);
		return true;
	}

	private Hashtable<String, String> readUserFile() throws LoginException {
		Hashtable<String, String> result = new Hashtable<>();
		try (BufferedReader userFile = new BufferedReader(new FileReader(userFileName));) {
			String line;
			while ((line = userFile.readLine()) != null) {
				String[] splittedLine = line.split(":");
				result.put(splittedLine[0], splittedLine[1]);
			}
		} catch (IOException e) {
			throw new LoginException("Fehler beim ffnen der Datei " + userFileName);
		}
		return result;
	}

}
